package com.ascendfinalproject.warehouse.controllers;

import com.ascendfinalproject.warehouse.kafkaservice.Sender;
import com.ascendfinalproject.warehouse.models.Session;
import com.ascendfinalproject.warehouse.services.SessionService;
import com.ascendfinalproject.warehouse.models.WarehouseOrderRequest;
import com.ascendfinalproject.warehouse.models.WarehouseOrderResponse;
import com.ascendfinalproject.warehouse.services.WarehouseOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

@RestController
@RequestMapping(value = "/api")
public class WarehouseController {

  @Autowired
  WarehouseOrderService orderService;

  @Autowired
  Sender sender;

//  @CrossOrigin
//  @GetMapping("/order/{id}")
//  public void getOrderToPublish(@PathVariable String id) {
//    Mono<WarehouseOrderResponse> record = orderService.getById(id);
//    record.map(e -> {
//      sender.sendOrder(e);
//      return e;
//    }).subscribe();
//  }


  @CrossOrigin
  @GetMapping(value = "/orders")
  public Flux<WarehouseOrderResponse> getOrders() {
    return orderService.getOrders();
  }


  @CrossOrigin
  @GetMapping("/orders/{id}")
  public Mono<ResponseEntity> getById(@PathVariable String id) {
    return orderService.getById(id)
            .map(order -> (ResponseEntity.status(HttpStatus.OK).body(order)))
            .cast(ResponseEntity.class)
            .defaultIfEmpty(ResponseEntity.status(HttpStatus.NOT_FOUND).body(null));
  }

  @CrossOrigin
  @PostMapping(value = "/kafkaOrders")
  public void createOrderKafka(@Valid @RequestBody WarehouseOrderRequest order) {
    // STEP ONE
    sender.sendOrder(order);
//    return orderService.createOrder(order);
  }

  @CrossOrigin
  @PostMapping(value = "/orders")
  public Mono<WarehouseOrderResponse> createOrder(@Valid @RequestBody WarehouseOrderRequest order) {
    return orderService.createOrder(order);
  }

  @CrossOrigin
  @PutMapping(value = "/orders/{id}")
  public Mono<WarehouseOrderResponse> updateOrder(@RequestBody WarehouseOrderResponse order, @PathVariable(value = "id") String id) {
    return orderService.updateOrder(order, id);
  }

  @CrossOrigin
  @DeleteMapping(value = "/orders/{id}")
  public Mono<Void> deleteOrder(@PathVariable(value = "id") String id) {
    return orderService.deleteOrder(id);
  }

  @CrossOrigin
  @DeleteMapping(value = "/orders/all")
  public Mono<Void> deleteAllOrders() {
    System.out.println("Deleted all orders");
    return orderService.deleteAllOrders();
  }

}
