package com.ascendfinalproject.warehouse.services;

import com.ascendfinalproject.warehouse.exceptions.NotFoundException;
import com.ascendfinalproject.warehouse.models.Address;
import com.ascendfinalproject.warehouse.models.WarehouseOrderRequest;
import com.ascendfinalproject.warehouse.models.WarehouseOrderResponse;
import com.ascendfinalproject.warehouse.repositories.WarehouseOrderRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Date;

@Service
public class WarehouseOrderService {

    private String RECEIVED = "RECEIVED";
    private String FULFILLED = "FULFILLED";
    private String CANCELLED = "CANCELLED";

  @Autowired
  WarehouseOrderRepository orderRepository;

  public Mono<WarehouseOrderResponse> getById(String id) {
    return orderRepository.findById(id);
  }

  public Flux<WarehouseOrderResponse> getOrders() {
    return orderRepository.findAll();
  }

  public Mono<WarehouseOrderResponse> createOrder(WarehouseOrderRequest order) {
    System.out.println("coming in from receiver " + order);

      WarehouseOrderResponse response = new WarehouseOrderResponse();

      response.setOrderId(order.getId());
      response.setStatus(RECEIVED);
      response.setCreatedAt(new Date(order.getOrderCreatedAt()));
      response.setOrderItems(order.getOrderItems());
      Address address = order.getCustomerAddress();
      response.setAddress(address.getStreet() + ", " +
                          address.getCity() + ", " +
                          address.getState() + ", " +
                          address.getZip()
      );
      return orderRepository.save(response);
  }


  public Mono<WarehouseOrderResponse> updateOrder(WarehouseOrderResponse order, String id) {
    return orderRepository.findById(id)
        .flatMap(existingOrder -> {
            if (existingOrder.getStatus().equals(RECEIVED)) {
                if (order.getStatus().equals(FULFILLED) || order.getStatus().equals(CANCELLED)) {
                    existingOrder.setStatus(order.getStatus());
                    existingOrder.setModifiedAt(new Date(System.currentTimeMillis()));
                }
            }
            return orderRepository.save(existingOrder);
        });


  }

  public Mono<Void> deleteOrder(String id) {
    return orderRepository.deleteById(id);
  }

  public Mono<Void> deleteAllOrders() {
    return orderRepository.deleteAll();
  }

}
